<strong>
<a href="{% url 'admin:dbpref_edit_form' instance.pk %}" title="edit form: {{ instance.site }} - {{ instance.app_label }} - {{ instance.form_name }}">
    {{ instance.form_name }}
</a>
</strong>